from ..repositories.json.classes import RuntimeJSON
from .static_execution import StaticExecution
from .execution import RequestData


class ScriptExecution(StaticExecution):
    @staticmethod
    def create_with_stage_run(runtime_json: RuntimeJSON, stage_run_id: str):
        execution = ScriptExecution(runtime_json, is_initial=False)
        execution.init_stage_run(stage_run_id)
        return execution

    def __init__(self, runtime_json: RuntimeJSON, is_initial: bool):
        request = RequestData(
            headers={},
            body="",
            method="GET",
            query_params={},
        )

        super().__init__(runtime_json, is_initial, request)

    def setup_context(self, request: RequestData):
        # TODO: This was added to allow script to run without a stage run on tests
        if self.stage_run is None:
            super().init_stage_run()



from .ClickInput import ClickInput
from .TextOutput import TextOutput
from .TextareaInput import TextareaInput
from .LinkOutput import LinkOutput
from .NumberInput import NumberInput
from .CurrencyInput import CurrencyInput
from .FileOutput import FileOutput
from .ImageInput import ImageInput
from .VideoInput import VideoInput
from .PasswordInput import PasswordInput
from .TagInput import TagInput
from .CustomInput import CustomInput
from .TimeInput import TimeInput
from .NumberSliderInput import NumberSliderInput
from .MarkdownOutput import MarkdownOutput
from .DateInput import DateInput
from .CardsInput import CardsInput
from .PlotlyOutput import PlotlyOutput
from .HtmlOutput import HtmlOutput
from .IframeOutput import IframeOutput
from .CheckboxInput import CheckboxInput
from .ImageOutput import ImageOutput
from .CpfInput import CpfInput
from .CnpjInput import CnpjInput
from .DropdownInput import DropdownInput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .PhoneInput import PhoneInput
from .KanbanBoardInput import KanbanBoardInput
from .ListInput import ListInput
from .PandasOutput import PandasOutput
from .ToggleInput import ToggleInput
from .TextInput import TextInput
from .FileInput import FileInput
from .EmailInput import EmailInput
from .RatingInput import RatingInput
from .RichTextInput import RichTextInput
from .MultipleChoiceInput import MultipleChoiceInput
from .CodeInput import CodeInput
from .LatexOutput import LatexOutput
from .ProgressOutput import ProgressOutput
from .ChecklistInput import ChecklistInput
from .NpsInput import NpsInput



from abstra_internals.constants import get_persistent_dir  # exported

from abstra_internals.flows import (
    AuthResponse,
    get_user,
    execute_js,
    alert,
    redirect,
    get_query_params,
)  # exported



from .generated.outputs import display_plotly


def _overload_plotly_show():
    try:
        from plotly.io import renderers
        from plotly.graph_objects import Figure
        from plotly.io._base_renderers import ExternalRenderer

        class FormsRenderer(ExternalRenderer):
            def render(self, fig_dict):
                fig = Figure(data=fig_dict)
                display_plotly(fig)
                return {}

        renderers["abstra-forms"] = FormsRenderer()
        renderers.default = "abstra-forms"
    except Exception as e:
        pass



import os

HOST = os.getenv("ABSTRA_HOST", "localhost")
DEFAULT_PORT = os.getenv("PORT") or os.getenv("ABSTRA_SERVER_PORT")
ENABLE_TALISMAN = os.getenv("ABSTRA_ENABLE_TALISMAN", "false").lower() == "true"

IS_PREVIEW = os.getenv("ABSTRA_ENVIRONMENT") != "production"
BUILD_ID = os.getenv("ABSTRA_BUILD_ID") or "dev"

CLOUD_API_ENDPOINT = os.getenv("CLOUD_API_ENDPOINT") or "https://cloud-api.abstra.cloud"

LOCAL_CONNECTORS_EXECUTE_URL = "https://cloud-api.abstra.cloud/cli/connectors/execute"
CONNECTORS_EXECUTE_URL = os.getenv("ABSTRA_CONNECTORS_EXECUTE_URL")

LOCAL_TABLES_EXECUTE_URL = "https://cloud-api.abstra.cloud/cli/tables/execute"
TABLES_EXECUTE_URL = os.getenv("ABSTRA_TABLES_EXECUTE_URL")

SIDECAR_SHARED_TOKEN = os.getenv("ABSTRA_SIDECAR_SHARED_TOKEN", "")
LOG_SERVICE_URL = os.getenv("ABSTRA_LOG_SERVICE_URL")
PUBLIC_KEY = os.getenv("ABSTRA_JWT_PUBLIC_KEY_PEM")
SIDECAR_URL = os.getenv("ABSTRA_SIDECAR_URL")
AUTHN_URL = os.getenv("ABSTRA_AUTHN_URL")

FILES_FOLDER = os.getenv("ABSTRA_FILES_FOLDER")



from ..widget_base import Input
from dataclasses import dataclass
from typing import List, Optional
from uuid import uuid4 as uuid


@dataclass
class KanbanStage:
    key: str
    label: str

    @staticmethod
    def make(data):
        if isinstance(data, str):
            return KanbanStage(key=data, label=data)
        elif isinstance(data, dict):
            return KanbanStage(
                key=data.get("key", str(uuid())), label=data.get("label", None)
            )
        elif isinstance(data, KanbanStage):
            return data

    def to_dict(self):
        return {"key": self.key, "label": self.label}


@dataclass
class KanbanTag:
    key: str
    label: str

    @staticmethod
    def make(data):
        if isinstance(data, str):
            return KanbanTag(key=data, label=data)
        elif isinstance(data, dict):
            return KanbanTag(
                key=data.get("key", str(uuid())), label=data.get("label", None)
            )
        elif isinstance(data, KanbanTag):
            return data
        raise ValueError("Invalid tag data")

    def to_dict(self):
        return {"key": self.key, "label": self.label}


@dataclass
class KanbanCard:
    key: str
    title: Optional[str]
    text: Optional[str]
    tags: List[KanbanTag]
    stage_key: Optional[str]

    @staticmethod
    def make(data):
        if isinstance(data, str):
            return KanbanCard(key=data, title=data, text=None, tags=[], stage_key=None)
        elif isinstance(data, dict):
            return KanbanCard(
                key=data.get("key", str(uuid())),
                title=data.get("title", None),
                text=data.get("text", None),
                tags=list(map(KanbanTag.make, data.get("tags", []))),
                stage_key=data.get("stageKey", None),
            )
        elif isinstance(data, KanbanCard):
            return data

    def to_dict(self):
        return {
            "key": self.key,
            "title": self.title,
            "text": self.text,
            "tags": [tag.to_dict() for tag in self.tags],
            "stageKey": self.stage_key,
        }


class KanbanBoardInput(Input):
    type = "kanban-board-input"

    def __init__(self, key: str, **kwargs):
        super().__init__(key)
        self.set_props(kwargs)

    def set_props(self, kwargs):
        self.label = kwargs.get("label", "")
        self.disabled = kwargs.get("disabled", False)
        self.stages = list(
            map(
                KanbanStage.make,
                kwargs.get(
                    "stages",
                    [
                        KanbanStage.make("To-Do"),
                        KanbanStage.make("Doing"),
                        KanbanStage.make("Done"),
                    ],
                ),
            )
        )
        self.set_value(kwargs.get("initial_value", []))

    def set_value(self, value):
        self.value = list(map(KanbanCard.make, value or []))

    def serialize_value(self):
        if self.value is None:
            return []
        return [(card.to_dict() if card else None) for card in self.value]

    def render(self, context: dict):
        stages = [(stage.to_dict() if stage else None) for stage in self.stages]
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "value": self.serialize_value(),
            "stages": stages,
            "disabled": self.disabled,
            "errors": self.errors,
        }



from __future__ import annotations
import traceback
from typing import Optional, TYPE_CHECKING, Tuple, Dict, Union
import flask_sock
from ..contract import Message, StdioMessage, should_send
from ..utils import deserialize, serialize
from .execution import Execution, RequestData, NoExecutionFound
from simple_websocket.ws import ConnectionClosed

if TYPE_CHECKING:
    from ..repositories.json.classes import DashJSON, FormJSON


class LiveExecution(Execution):
    type = "execution"
    _connection: flask_sock.Server

    @staticmethod
    def get_execution() -> Optional["LiveExecution"]:
        execution = Execution.get_execution()
        if isinstance(execution, LiveExecution):
            return execution

        return None

    @staticmethod
    def broadcast(msg: Message):
        for e in list(Execution.executions.values()):
            if isinstance(e, LiveExecution) and e.connected and e.closed == False:
                e.send(msg)

    def __init__(
        self,
        runtime_json: Union["FormJSON", "DashJSON"],
        is_initial: bool,
        connection: flask_sock.Server,
        request: RequestData,
        execution_id=None,
    ):
        self._connection = connection
        if execution_id is not None:
            self.id = execution_id

        super().__init__(runtime_json, is_initial, request, execution_id=execution_id)

    def send(self, msg: Message):
        self.log(msg.type, msg.data)

        if not should_send(msg, self.is_preview):
            return

        str_data = serialize(msg.to_json(self.is_preview))
        self._connection.send(str_data)

    def recv(self) -> Tuple[str, Dict]:
        try:
            str_data = self._connection.receive()
        except ConnectionClosed:
            self.close()
            return "close", {}

        data = deserialize(str_data)
        self.log(data["type"], data)
        return data["type"], data

    def close(self, reason: Optional[str] = ""):
        self._connection.close(message=reason)
        self.closed = True

    def stdio(self, type: str, text: str):
        self.send(StdioMessage(type, text))
        return super().stdio(type, text)

    @property
    def connected(self) -> bool:
        return self._connection.connected

    def end(self):
        if self.connected:
            self.close(reason=traceback.format_exc() or None)


def get_live_execution_throwable() -> LiveExecution:
    execution = LiveExecution.get_execution()
    if not execution:
        raise NoExecutionFound
    return execution



from abstra_internals.contract import Message
from abstra_internals.execution.live_execution import get_live_execution_throwable


def send(message: Message):
    execution = get_live_execution_throwable()
    return execution.send(message)


def receive():
    execution = get_live_execution_throwable()
    while True:
        type, data = execution.recv()
        if type in ["heartbeat", "browser:try-disconnect"]:
            continue

        return data



import flask, flask_cors, flask_talisman

from .overloads import overloads
from .controller import MainController
from .apps.player import get_player_bp
from ..settings import SettingsController
from ..utils.environment import ENABLE_TALISMAN


# USED ON CLOUD SERVER
def get_cloud_app(root: str):
    SettingsController.set_root_path(root)
    SettingsController.set_server_port(None)

    overloads()
    controller = MainController()
    app = flask.Flask(__name__)
    flask_cors.CORS(app)

    if ENABLE_TALISMAN:
        talisman = flask_talisman.Talisman(app, content_security_policy=None)

        @app.route("/_healthcheck")
        @talisman(force_https=False)
        def _healthcheck():
            return "ok"

    else:

        @app.route("/_healthcheck")
        def _healthcheck():
            return "ok"

    player = get_player_bp(controller)
    app.register_blueprint(player)

    return app
